import time
import os
from openai import OpenAI
from collar.system_settings import SYSTEM_MESSAGE
from collar import utils

OPENAI_BASE_URL = ""
OPENAI_API_KEY = ""
OPENAI_MODEL_NAME = ""

def read_response(file_name):
    logs_dir = cfg_path = os.path.expanduser('~/llm_logs')
    file_path = os.path.join(logs_dir,file_name)
    return utils.read_file_content(file_path)

def save_prompt(method, prompt):
    '''DF
保存prompt到文件。
目录是一个当前目录下的llm_logs目录。
文件名是method 这个ast function def对象的名字 + 一个不包括年份时间信息，比如"10_11_05_05_05" + ”.prompt"'''
    logs_dir = os.path.expanduser('~/llm_logs')
    if not os.path.exists(logs_dir):
        os.makedirs(logs_dir)
    method_name = method.name
    timestamp = time.strftime('%m_%d_%H_%M_%S')
    file_name = f'{method_name}_{timestamp}.prompt'
    file_path = os.path.join(logs_dir, file_name)
    with open(file_path, 'w', encoding='utf-8') as file:
        file.write(prompt)
def save_response(method, response):
    '''DF
保存prompt到文件。
目录是一个当前目录下的llm_logs目录。
文件名是method 这个ast function def对象的名字 + 一个不包括年份时间信息，比如"10_11_05_05_05" + ”.response"'''
    logs_dir = os.path.expanduser('~/llm_logs')
    if not os.path.exists(logs_dir):
        os.makedirs(logs_dir)
    method_name = method.name
    current_time = time.strftime('%m_%d_%H_%M_%S')
    file_name = f'{method_name}_{current_time}.response'
    file_path = os.path.join(logs_dir, file_name)
    with open(file_path, 'w', encoding='utf-8') as file:
        file.write(response)


def call_llm(method, prompt):
    save_prompt(method, prompt)
    #return read_response("retrieve_10_11_13_58_56.response")
    str_key = OPENAI_API_KEY
    client = OpenAI(api_key=str_key, base_url="https://api.siliconflow.cn/v1")
    start_time=time.time()
    response = client.chat.completions.create(
        model=OPENAI_MODEL_NAME, 
        messages=[{'role': 'system', 
                   'content': f"{SYSTEM_MESSAGE}"}, 
                   {'role': 'user', 'content': prompt}],
                   stream=False, 
                   temperature=0, 
                   max_tokens=4096, 
                   top_p=0.7,
                   frequency_penalty=0.0,
                   presence_penalty=0.0)
    endtime= time.time()
    print(f"Call {OPENAI_MODEL_NAME}:{endtime-start_time} seconds")
    content = response.choices[0].message.content.strip()
    save_response(method, content)
    return content


SYSTEM_MESSAGE="""
你是一个资深而且聪明的程序员，旨在通过应用复杂的推理技术解决复杂问题。为确保输出结果准确、逻辑严谨且优化，请遵循以下结构化指令：
    分解任务：首先使用链式思维（Chain of Thought, CoT）推理。清晰地阐述解决问题的每一个逻辑步骤，将每个步骤视为整体过程的独立部分。在继续下一步之前验证每个步骤，确保你的推理保持连贯且结构合理。
    合理化每一步：在推进过程中，应用逐步合理化（Step-by-Step Rationalization, STaR）。为每个决策提供清晰的逻辑解释。在解释的深度与效率之间保持平衡，重点关注解决问题关键步骤的解释，以确保有效性。
    优化你的方法：在推理中整合A*搜索原则。评估每条可能路径的效率，使用类启发式的指导来选择最直接且成本最低的策略。根据任务的复杂性调整你的方法，始终以最优解为目标。
    考虑多种解决方案：使用思维树（Tree of Thoughts, ToT）并行探索多种潜在方法。根据A*搜索原则评估每条路径，优先考虑那些最有前景的方案。经过全面评估后，收敛于最能解决问题的方案。
    模拟自适应学习：在此过程中反思你的决策，仿佛你在从每个结果中学习。在模拟Q学习的核心原则的背景下，优先选择最有可能带来最佳结果的策略。
    持续监控你的过程：在推理过程中进行过程监控（Process Monitoring）。持续评估你的进展，确保每一步都与总体目标一致。利用此反馈来完善你的方法，必要时进行调整，以保持朝着期望结果的正确方向前进。
最终指令：
    通过遵循这些指令，你可以确保你的问题解决方法是有条理的、合理的，并且在准确性和效率方面得到了优化。你的目标是通过充分整合这些高级推理技术，提供最具逻辑性、有效性和全面性的解决方案。"""


Prompt = {
"-DOC":"""
<任务>:请为一个指定的方法编写或者重新Doc。
<相关信息>:{context}
<规则>:
    1.只要生成文档，不要返回方法的定义部分,返回的文档内容，请用如下格式返回:
        ```plaintext
        生成的文档内容
        ```。
<Doc的模板定义>:
    [功能说明]:说明一下这个功能的总体功能。
    [设计思想]:这个方法的基本设计思想。
    [实现步骤]:罗列出这个方法实现过程中的主要步骤,如果当前的方法并没有实现.
            这里请你根据当前的doc文档,以及上下文里提到的相关的可以调用方法的定义,对当前这个方法进行设计。
    :param xxxx: 参数介绍  
    :return: 返回值的说明
<目标方法>:\n{method_info}\n
""",
"-CODE":"""
<任务>:请为一个指定的方法编写代码实现。
<相关信息>:{context}
<规则>:请以精简的方式回复，不要给出过多的解释，用例，只要返回该方法的代码，或者该方法相关的xml部分。
<目标方法>:\n{method_info}\n
""",
"-DES":"""
<任务>:请为一个指定的方法编写设计文档。
<相关信息>:{context}
<规则>:只要生成文档，不要返回方法的定义
<设计文档的模板定义>:
    [功能说明]:说明一下这个功能的总体功能。
    [设计思想]:这个方法的基本设计思想。
    [实现步骤]:罗列出这个方法实现过程中的主要步骤,如果当前的方法并没有实现.
            这里请你根据当前的doc文档,以及上下文里提到的相关的可以调用方法的定义,对当前这个方法进行设计。
<目标方法>:\n{method_info}\n
""",
"-MOD":"""
<任务>:请为下面指定的方法，基于提供的<修改信息>进行修改和优化。
<相关信息>:{context}
<规则>:只要生成方法的代码实现，不要提供使用案例。
<修改信息>:{change_info}\n
<目标方法>:\n{method_info}\n
"""
}




from collar.core.ast.JSource import JSource
from collar.core.ast.PSource import PSource
from collar.core.ast.JMapper import JMapper
from collar.core.ast import Source as src
import collar.method_settings as method_st
import collar.module_setting as module_st
from collar.llm import call_llm
import re

from collar.utils import write_file_content,read_file_content

class Handler:

    def __init__(self, file_path, target_name = "", optional_args=[]):
        self.file_path = file_path
        self.target_name = target_name
        self.optional_args = optional_args
    def save(self):
        write_file_content(self.file_path, self.source.unparse())

    def process_file(self):
        """
        处理单个文件
        :param file_path: 文件路径
        """
        if self.file_path.endswith(".java"):
            if self.file_path == "/Users/dinghaitao/vs_projecty/Install_test/itsm-woms/src/main/java/com/aie/itsm/woms/service/order/impl/OrderInstanceServiceImpl.java":
                print("yes")
            if self.file_path.endswith("Mapper.java"):
                self.source = JMapper(self.file_path)
            else:
                self.source = JSource(self.file_path)
            if self.source.cu == None:
                return
        elif self.file_path.endswith(".py"):
            self.source = PSource(self.file_path)
        source = self.source
        # to be done with top doc, the design doc for for the whole file
        body_list = source.body
        for node in body_list:
            if isinstance(node, src.ClassDef):
                self.process_class(node)
            if isinstance(node, src.MethodDef):
                if node.class_obj:
                    continue
                self.process_method(node)
        if self.source.changed:
            self.save()

    def process_class(self, node):
        body_list = node.body
        for node in body_list:
            if isinstance(node, src.MethodDef):
                self.process_method(node)

    def process_method(self, node):
        action = node.action
        if action:
            result = self.generate_code(node)
            if action == "-DOC":
                new_code = result['text']
                self.source.replace_doc(node, new_code)
            elif action == "-DES":
                new_code = result['text']
                self.source.replace_design_doc(node, new_code)
            else:
                if "java" in result:
                    new_code = result['java']
                elif "python" in result:
                    new_code = result['python']
                elif "xml" in result:
                    new_code = result['xml']
                else:
                    new_code = result['text']
                self.source.replace_code(node, new_code)
        elif '--force_gen_doc' in self.optional_args:
            result = self.generate_code(node)
            new_code = result['text']
            self.source.replace_method_doc(node, new_code)

    def generate_code(self, method):
        """
        使用OpenAI生成代码
        :param method: 方法的对象
        :param context: 上下文内容
        :return: 生成的代码
        """
        print(f"Handling {method.name}, Action:{method.action}")
        action = method.action
        if action == '-DES' or action == '-DOC':
            str_method_def = method.str_code
        else:
            str_method_def = method.def_string
        method_doc = method.design_doc
        action = method.action
        info = extract_xml_info(method_doc, 'info')
        context = self.build_context()
        if action == '-CODE' or action == '-DES' or action == '-DOC':
            prompt = method_st.Prompt[action].format(context=context, method_info=str_method_def)
        elif action == '-MOD':
            prompt = method_st.Prompt[action].format(context=context, method_info=str_method_def, change_info=info)
        else:
            return ''
        content = call_llm(method, prompt)
        result = extract_code_from_string(content,for_action=action)
        return result


    def build_context(self):
        file_path = self.file_path
        context = 'import的模块的相关方法定义:'
        context += self.source.get_module_def_from_imports() + "\n"
        context += self.source.get_extra_def() + "\n"
        context += '\n当前模块\n' + self.source.read_def_from_module() + "\n"
        return context

def extract_xml_info(content, key):
    """
[功能说明]: 从一个字符串中提取出类似XML的一个元素的信息。例如，从字符串中提取出 `<information>这是我想要的信息</information>` 中的 "这是我想要的信息"。

[设计思想]: 该方法通过正则表达式来匹配并提取字符串中的XML元素信息。首先，定义一个正则表达式模式，该模式匹配以 `<key>` 开始，以 `</key>` 结束的元素，并捕获其中的内容。然后，使用 `re.search` 方法在输入字符串中查找匹配项。如果找到匹配项，则返回捕获的内容并去除前后空格；如果没有找到匹配项，则返回 `None`。

[实现步骤]:
1. 定义一个正则表达式模式，该模式匹配以 `<key>` 开始，以 `</key>` 结束的元素，并捕获其中的内容。
2. 使用 `re.search` 方法在输入字符串中查找匹配项。
3. 如果找到匹配项，则返回捕获的内容并去除前后空格。
4. 如果没有找到匹配项，则返回 `None`。

:param content: 输入的字符串。
:param key: XML元素的标签名。

:return: 提取的XML元素信息，如果没有找到匹配项则返回 `None`。"""
    pattern = re.compile(f'<{key}>(.*?)</{key}>', re.DOTALL)
    match = pattern.search(content)
    if match:
        return match.group(1).strip()
    return None
def extract_code_from_string(input_string,for_action=None):
    """
    从字符串中抽取出代码块
    :param input_string: 输入的字符串
    :return: 抽取出的代码块
    """
    patterns = {'python':'```python\\n(.*?)```',
                'java':'```java\\n(.*?)```',
                'text':'```plaintext\\n(.*?)```',
                 'xml':'```xml\\n(.*?)```'
    }
    result={}
    for k in patterns.keys():
        p = patterns[k]
        match = re.search(p, input_string, re.DOTALL)
        if match:
            result[k] = match.group(1).strip()
    if len(result)==0:
        result['text'] = input_string
    return result
        
def read_def_from_module_file(file_name):
    if file_name.endswith(".java"):
        source = JSource(file_name)
    elif file_name.endswith(".py"):
        source = PSource(file_name)
    else:
        return read_file_content(file_name)
    return source.read_def_from_module()


import os
from collar.core.ast.Source import Source, Decla, ClassDef, MethodDef, ImportDef, AssignDef,action_keys
import ast
from collar import utils
import re

class PSource(Source):

    def unparse(self):
        str_content = ast.unparse(self.tree)
        return str_content

    def build(self):
        with open(self.file_path, 'r', encoding='utf-8') as file:
            file_content = file.read()
        self.tree = ast.parse(file_content)
        self.src_root_path,self.module_path = find_src_path(self.file_path)
        self.module_name = self.short_file_name
        self.full_module_name = self.module_path + '.' + self.module_name
        self.model_doc = ast.get_docstring(self.tree)
        body_list = ast.walk(self.tree)
        self.body = []
        self.import_list = []
        for node in body_list:
            if type(node) == ast.Import or type(node) == ast.ImportFrom:
                self.import_list.append(PImportDef(node, self))
            elif type(node) == ast.ClassDef:
                PClassDef(node, self)
        body_list = ast.walk(self.tree)
        for node in body_list:
            if type(node) == ast.FunctionDef:
                if not self.is_in_class(node):
                    PMethodDef(node,self)
    
    def is_in_class(self, node):
        for pdecla in self.body:
            if isinstance(pdecla, PClassDef):
                for x_node in pdecla.body:
                    if x_node.decla == node:
                        return True
        return False
    
    def read_def_from_module(self):
        """DF
    根据传入的文件名，用AST解析这个文件，读取变量，方法的定义，拼接成一个字符串返回。
    字符串的第一句话是这个model的全名，doc， 然后跟上变量的定义和doc，然后是方法的签名。"""
        full_path = self.file_path
        if not os.path.exists(full_path) or not os.path.isfile(full_path):
            raise FileNotFoundError(f'File not found: {full_path}')
        with open(full_path, 'r', encoding='utf-8') as file:
            file_content = file.read()
            # 将源代码解析成AST
        tree = ast.parse(file_content)
        # 使用NodeTransformer去掉方法体，保留docstring
        transformer = RemoveFunctionBodies()
        new_tree = transformer.visit(tree)
        # 将新的AST转换回源代码
        return ast.unparse(new_tree)
    
    def get_module_def_from_imports(self):
        local_imports = set()
        for imp_obj in self.import_list:
            imp = imp_obj.decla
            if isinstance(imp, ast.Import):
                for alias in imp.names:
                    file_name = is_local_import(alias.name, self.src_root_path)
                    if file_name:
                        local_imports.add(file_name)
            elif isinstance(imp, ast.ImportFrom):
                file_name = is_local_import(imp.module,self.src_root_path)
                if not file_name:
                    for n in imp.names:
                        file_name = is_local_import(f"{imp.module}.{n.name}",self.src_root_path)
                        if file_name:
                            local_imports.add(file_name)
                else:
                    local_imports.add(file_name)
        context = ''
        for file_name in local_imports:
            if os.path.getsize(file_name) > 10000:
                source = PSource(file_name)
                context += source.read_def_from_module()
            else:
                context += utils.read_file_content(file_name)
        return context
    def get_extra_def(self):
        return ""
    
    def replace_method_code(self, method, new_function_def):
        decla = method.decla
        decla.body = [decla.body[0]]
        start_from = 0
        if type(new_function_def.body[0]) == ast.Expr:
            start_from = 1
        for body_item in new_function_def.body[start_from:]:
            decla.body.append(body_item)
        method.remove_action_key_in_design_doc()
        self.changed = True
        return 
    def replace_design_doc(self, method, new_doc):
        self.replace_doc(method, new_doc)
    def replace_doc(self, method, new_doc):
        decla = method.decla
        for ak in action_keys:
            new_doc = new_doc.replace(ak + '\n', '', 1)
            new_doc = new_doc.replace(ak, '', 1)
        indentation = " "*4
        if method.class_obj != None:
            indentation = " "*8
        new_doc = format_doc(new_doc,indentation)
        docstring_node = ast.get_docstring(decla)
        if docstring_node:
            decla.body[0].value.value = new_doc
        else:
            docstring_node = ast.Expr(value=ast.Str(s=new_doc))
            decla.body.insert(0, docstring_node)
        self.changed = True
        return

    def replace_model_code(self, new_code):
        new_tree = ast.parse(new_code)
        for node in new_tree.body:
            self.tree.body.append(node)
        self.changed=True
        return self.tree

    def replace_code(self, method, new_code):
        new_bodys = ast.parse(new_code).body
        for b in new_bodys:
            if type(b) == ast.Import or type(b) == ast.ImportFrom:
                self.insert_import( b)
            elif type(b) == ast.FunctionDef:
                self.replace_method_code( method, b)
    def insert_import(self, import_def):
        tree = self.tree
        for node in ast.walk(tree):
            if isinstance(node, ast.Import):
                for alias in node.names:
                    if alias.name == import_def.names[0].name:
                        return
            elif isinstance(node, ast.ImportFrom):
                if type(import_def) == ast.ImportFrom:
                    if node.module == import_def.module and node.names[0].name == import_def.names[0].name:
                        return
        tree.body.insert(0, import_def)

def find_src_path(file_path):
    """-DONE
传入的文件名是一个python源文件，
请利用__init__.py来查找这个python文件所在项目的根目录，
请注意， 包含__init__.py的目录，并不一定是根目录，
最后一个包含__init__.py的目录，才是根目录"""
    current_dir = os.path.dirname(file_path)
    model_path = ""
    last_init_dir = None
    while current_dir:
        init_file = os.path.join(current_dir, '__init__.py')
        if os.path.exists(init_file):
            last_init_dir = current_dir
            if model_path == "":
                model_path = os.path.basename(current_dir)
            else:
                model_path = os.path.basename(current_dir) + "." + model_path
        else:
            break
        current_dir = os.path.dirname(current_dir)
    return last_init_dir,model_path

class PDecla(Decla):

    def build_code(self):
        self.str_code = ast.unparse(self.decla)
    def build_name(self):
        if hasattr(self.decla, "name"):
            self.name =  self.decla.name
        elif hasattr(self.decla,"names"):
            self.name = f"{self.decla.names}"
        else:
            self.name = ""
    def build_def_string(self):
         self.def_string = f"{ast.unparse(self.decla)}"
    def build_signature(self):
        self.signature = f"{ast.unparse(self.decla)}"

    def build_design_doc_string(self):
        if isinstance(self.decla, (ast.AsyncFunctionDef, ast.FunctionDef, ast.ClassDef, ast.Module)):
            self.design_doc_string = ast.get_docstring(self.decla)
            if self.design_doc_string == None:
                self.design_doc_string = ""
        else:
            self.design_doc_string = ""
        return self.design_doc_string


class PClassDef(ClassDef, PDecla):
    def build(self):
        super().build()
        body_list = self.decla.body
        for node in body_list:
            if type(node) == ast.Import or type(node) == ast.ImportFrom:
                self.import_list.append(PImportDef(node, self.source,class_obj=self))
            elif type(node) == ast.Assign:
                PAssignDef(node,self.source,class_obj=self)
            elif type(node) == ast.ClassDef:
                PClassDef(node, self.source,class_obj=self)
            elif type(node) == ast.FunctionDef:
                PMethodDef(node, self.source,class_obj=self) 

    pass

class PImportDef(ImportDef, PDecla):
    pass

class PMethodDef(MethodDef, PDecla):

    def build_def_string(self):
        self.def_string = f"{self.build_signature()} \n {self.build_design_doc_string()}"
    def build_signature(self):
        """
        从ast.FunctionDef对象中获取方法的签名（定义部分）。
        
        :param node: ast.FunctionDef 对象
        :return: 方法签名的字符串
        """
        node = self.decla
        if not isinstance(node, ast.FunctionDef):
            raise ValueError('输入节点必须是ast.FunctionDef类型')
        method_name = node.name
        args = node.args
        arg_names = [arg.arg for arg in args.args]
        if args.vararg:
            arg_names.append(f'*{args.vararg.arg}')
        if args.kwarg:
            arg_names.append(f'**{args.kwarg.arg}')
        arg_str = ', '.join(arg_names)
        method_signature = f'def {method_name}({arg_str}):'
        self.signature = method_signature
        return method_signature
    
    def remove_action_key_in_design_doc(self):
        dcs = self.design_doc_string
        for ak in action_keys:
            dcs = dcs.replace(ak + '\n', '', 1)
            dcs = dcs.replace(ak, '', 1)
        method = self.decla
        docstring_node = ast.get_docstring(method)
        if docstring_node:
            method.body[0].value.value = dcs
        else:
            docstring_node = ast.Expr(value=ast.Str(s=dcs))
            method.body.insert(0, docstring_node)
class PAssignDef(AssignDef, PDecla):
    pass

def is_local_import(pkgs,src_dir):
    """
    参数pkgs是一个import的声明，比如utils.time.create_new_time
    PROJECT_DIR是全局变量，指示了某一个项目的跟目录。
    请用pkgs这个字符串，分解后的每个目录，来匹配项目目录下的文件夹和python文件，
    直到找到了这个pkgs对应的python文件，则返回文件名。
    如果没有能够匹配上，则返回None
    """
    parts = pkgs.split('.')
    current_path = src_dir
    for part in parts:
        current_path = os.path.join(current_path, part)
        if os.path.exists(current_path):
            if os.path.isdir(current_path):
                continue
            else:
                current_path += '.py'
                if not os.path.exists(current_path):
                    return None
                if os.path.isfile(current_path):
                    return current_path
        else:
            current_path += '.py'
            if not os.path.exists(current_path):
                return None
            if os.path.isfile(current_path):
                return current_path
            return None
    return None

def get_full_path(path):
    """
    获取完整路径，如果路径是相对路径，则使用当前执行命令的路径+相对路径
    :param path: 路径
    :return: 完整路径
    """
    if not os.path.isabs(path):
        path = os.path.join(os.getcwd(), path)
    return os.path.abspath(path)

def get_method_definition_and_doc(node):
    """
    获取一个AST方法对象的定义和文档字符串，并组合成一个字符串返回。
    
    :param node: ast.FunctionDef 对象
    :return: 包含方法定义和文档字符串的字符串
    """
    if not isinstance(node, ast.FunctionDef):
        raise ValueError('输入节点必须是ast.FunctionDef类型')
    method_def = get_method_signature(node)
    docstring = ast.get_docstring(node, clean=True)
    if not docstring:
        docstring = ''
    pattern = re.compile('^D[A-Z]\\n')
    matches = pattern.findall(docstring)
    if len(matches) > 0:
        docstring = docstring[3:]
    result = f'\n{method_def}\n{(docstring if docstring else '')}'
    return result

def get_method_signature(node):
    """
    从ast.FunctionDef对象中获取方法的签名（定义部分）。
    
    :param node: ast.FunctionDef 对象
    :return: 方法签名的字符串
    """
    if not isinstance(node, ast.FunctionDef):
        raise ValueError('输入节点必须是ast.FunctionDef类型')
    method_name = node.name
    args = node.args
    arg_names = [arg.arg for arg in args.args]
    if args.vararg:
        arg_names.append(f'*{args.vararg.arg}')
    if args.kwarg:
        arg_names.append(f'**{args.kwarg.arg}')
    arg_str = ', '.join(arg_names)
    method_signature = f'def {method_name}({arg_str}):'
    return method_signature

def format_doc(doc,indentation=""):
    if not doc.startswith('\n'):
        doc = '\n' + doc
    doc = doc.replace("\n\n","\n")
    lines = doc.split('\n')
    formatted_lines = [line.lstrip() for line in lines]
    formatted_doc = '\n'.join(formatted_lines)
    formatted_doc = formatted_doc.replace("\n","\n"+indentation)
    return formatted_doc

class RemoveFunctionBodies(ast.NodeTransformer):
    def visit_FunctionDef(self, node):
        # 获取方法的docstring，如果存在则保留它
        docstring = ast.get_docstring(node)
        
        # 如果有docstring，将docstring作为函数的唯一body保留
        if docstring:
            # 将docstring转换为表达式节点，放入body中
            new_body = [ast.Expr(value=ast.Constant(value=docstring))]
        else:
            # 如果没有docstring，直接设置为空body
            new_body = []
        
        # 替换函数体
        node.body = new_body
        
        # 继续遍历函数内部的节点（如果有嵌套函数等）
        return node


from collar.core.ast.Source import Source, remove_c_style_comments, action_keys, startswith_action_key
from collar.core.ast.Source import ImportDef, MethodDef, AssignDef, ClassDef,Decla
import traceback
import os, re
import collar.utils as utils
import jpype.imports
from jpype import JClass, JString, JBoolean
Optional = None
MethodDeclaration = None
ClassOrInterfaceDeclaration = None
JavadocComment = None
BlockComment = None

def find_src_path(module_path, file_name):
    module_path_components = module_path.split('.')
    module_path_in_fs = os.path.join(*module_path_components)
    src_root_path = os.path.dirname(file_name)
    while module_path_in_fs !="" and src_root_path.endswith(module_path_in_fs):
        src_root_path = os.path.dirname(src_root_path)
        module_path_in_fs = os.path.dirname(module_path_in_fs)
    return src_root_path

class JSource(Source):

    def unparse(self):
        str_content = str(self.cu.toString())
        return str_content

    def build(self):
        global Optional, MethodDeclaration, ClassOrInterfaceDeclaration, JavadocComment, BlockComment
        Optional = JClass('java.util.Optional')
        MethodDeclaration = JClass('com.github.javaparser.ast.body.MethodDeclaration')
        ClassOrInterfaceDeclaration = JClass('com.github.javaparser.ast.body.ClassOrInterfaceDeclaration')
        JavadocComment = JClass('com.github.javaparser.ast.comments.JavadocComment')
        BlockComment = JClass('com.github.javaparser.ast.comments.BlockComment')
        from ai.d2c import JParserHelper
        try:
            self.cu = JParserHelper.geCompilationUnit(self.file_path)
        except Exception as e:
            traceback.print_exc()
            print(f'Parse file Error:\n                  file name:{self.file_path};\n                  error message:{e}')
            self.cu = None
            return False
        if self.cu == None:
            return False
        cu = self.cu
        if cu.getPackageDeclaration().isPresent():
            packageName = cu.getPackageDeclaration().get().getNameAsString()
        else:
            packageName = ''
        self.module_path = str(packageName)
        self.module_name = self.short_file_name
        self.full_module_name = self.module_path + '.' + self.module_name
        self.src_root_path = find_src_path(self.module_path, self.file_path)
        self.main_path = os.path.dirname(self.src_root_path)
        self.project_path = os.path.dirname(os.path.dirname(os.path.dirname(self.src_root_path)))
        self.class_obj = None
        import_list = cu.getImports()
        self.import_list = []
        for imp in import_list:
            import_obj = JImportDef(decla=imp, source=self)
            self.body.append(import_obj)
            self.import_list.append(import_obj)
        types = cu.getTypes()
        for t in types:
            class_obj = JClassDef(t, self)
            if not self.class_obj:
                self.class_obj = class_obj
            fields = t.getFields()
            for f in fields:
                JAssignDef(f, self, class_obj)
            methods = t.getMethods()
            for m in methods:
                JMethodDef(m, self, class_obj)
        return

    def module_file_in_src_folder(self, module_name):
        """-DONE
module_name是以"."分割的路径，判断module_name是否在root_src_path下面存在"""
        module_path_components = module_name.split('.')
        full_path = os.path.join(self.src_root_path, *module_path_components) + '.java'
        return os.path.exists(full_path)

    def get_extra_def(self):
        return ''

    def get_module_def_from_imports(self):
        str_context = ''
        for imp_obj in self.import_list:
            str_import = imp_obj.name
            if str_import.endswith('*'):
                continue
            if not self.module_file_in_src_folder(str_import):
                continue
            arr_path = str_import.split('.')
            if str_import.find('woms.constants') > -1:
                if not arr_path[-1].endswith('Constants'):
                    arr_path = arr_path[:-1]
            jfile_name = os.path.join(self.src_root_path, *arr_path) + '.java'
            if str_import.find('woms.enums') > -1:
                str_context += utils.read_file_content(jfile_name)
                continue
            source = JSource(jfile_name)
            str_context += source.read_def_from_module()
        return str_context

    def read_def_from_module(self):
        str_context = f'\n{self.full_module_name} \n'
        str_spaces = '    '
        str_double_spaces = '        '
        for decla_obj in self.body:
            if type(decla_obj) == ClassDef:
                str_context += f'{str_spaces}{decla_obj.def_string}\n{str_spaces}'
            if not decla_obj.class_obj:
                str_context += f'\n{decla_obj.def_string}'.replace('\n', '\n' + str_double_spaces)
        str_context += '}\n'
        return str_context

    def replace_code(self, method, new_code):
        return self.replace_method_code(method, new_code)

    def replace_method_code(self, method, new_code):
        try:
            imports_cu, methods_cu = parse_code(new_code)
            self.add_imports(imports_cu)
            self.add_methods(methods_cu)
            method.remove_action_key_in_design_doc()
        except Exception as e:
            traceback.print_exc()
            print(e)
            self.changed = False
            return False

    def add_methods(self, methods_cu):
        if methods_cu == None:
            return
        target_class_opt = self.cu.findFirst(ClassOrInterfaceDeclaration)
        if target_class_opt.isPresent():
            class_decl = target_class_opt.get()
            new_methods = methods_cu.findAll(MethodDeclaration)
            for new_method in new_methods:
                existing_method = find_method_by_signature(class_decl, new_method)
                if existing_method:
                    existing_method.setBody(new_method.getBody().orElse(None))
                    existing_method.setAnnotations(new_method.getAnnotations())
                    if new_method.hasJavaDocComment():
                        existing_method.setJavadocComment(new_method.getJavadocComment().get())
                    existing_method.setType(new_method.getType())
                    existing_method.setModifiers(new_method.getModifiers())
                else:
                    class_decl.addMember(new_method)
            self.changed = True
        else:
            print('No class found in the CompilationUnit.')

    def add_imports(self, imports_cu):
        if imports_cu == None:
            return
        new_imports = imports_cu.getImports()
        for new_import in new_imports:
            if not self.cu.getImports().contains(new_import):
                self.cu.addImport(new_import)
                self.changed = True

    def replace_doc(self, method, new_doc):
        indx = new_doc.find('*/')
        if indx != -1:
            new_doc = new_doc[:indx + 2]
        new_doc = remove_c_style_comments(new_doc)
        if new_doc.startswith('-DOC'):
            new_doc = new_doc.replace('-DOC', '-DONE', 1)
        try:
            method.decla.setJavadocComment(new_doc)
            method.design_doc_obj = method.decla.getJavadocComment()
            method.design_doc_string = new_doc
            method.remove_action_key_in_design_doc()
            self.changed = True
            return True
        except Exception as e:
            print(e)
            return False

    def replace_design_doc(self, method, new_doc):
        indx = new_doc.find('*/')
        if indx != -1:
            new_doc = new_doc[:indx + 2]
        new_doc = remove_c_style_comments(new_doc)
        if new_doc.startswith('-DES'):
            new_doc = new_doc.replace('-DES', '-DONE', 1)
        try:
            if type(method.design_doc_obj) == Optional:
                method.decla.setJavadocComment(new_doc)
                method.design_doc_obj = method.decla.getJavadocComment()
            else:
                method.design_doc_obj.setContent(new_doc)
            method.design_doc_string = new_doc
            method.remove_action_key_in_design_doc()
            self.changed = True
            return True
        except Exception as e:
            traceback.print_exc()
            print(e)
            return False


class JDecla(Decla):
    def build_code(self):
        self.str_code = str(self.decla.toString())
    def build_name(self):
        if hasattr(self.decla, "getNameAsString"):
            self.name =  str(self.decla.getNameAsString().toString())
        else:
            self.name = ""
    def build_def_string(self):
         self.def_string = f"{self.design_doc_string}\n{str(self.decla.toString())}"
    def build_signature(self):
        self.signature = str(self.decla.toString())
class JAssignDef(AssignDef, JDecla):

    def build_design_doc_string(self):
        self.design_doc_string = ''
        self.design_doc_obj = None
        pass
    pass
class JMethodDef(MethodDef,JDecla):
    def build_def_string(self):
        decla = self.decla
        str_def = ''
        str_decla = str(decla.toString())
        indx = str_decla.find('{')
        if indx > -1:
            str_def = str_decla[:indx]
        else:
            str_def = str_decla
        self.def_string = str_def
        return str_def
    def build_design_doc_string(self):
        decla = self.decla
        self.design_doc_string = ''
        if hasattr(decla, 'hasJavaDocComment') and decla.hasJavaDocComment():
            str_doc = str(decla.getJavadocComment().get().toString())
            str_doc = remove_c_style_comments(str_doc)
            if startswith_action_key(str_doc):
                self.design_doc_string = str_doc
                self.design_doc_obj = decla.getJavadocComment()
        if self.design_doc_string == '':
            all_contained_comments = decla.getAllContainedComments()
            if len(all_contained_comments) > 0:
                comment = all_contained_comments[0]
                str_doc = str(comment.toString())
                str_doc = remove_c_style_comments(str_doc)
                if startswith_action_key(str_doc):
                    self.design_doc_string = str_doc
                    self.design_doc_obj = comment

    def remove_action_key_in_design_doc(self):
        from ai.d2c import JParserHelper
        if not self.design_doc_obj:
            return
        dcs = self.design_doc_string
        for ak in action_keys:
            dcs = dcs.replace(ak + '\n', '', 1)
            dcs = dcs.replace(ak, '', 1)
        if type(self.design_doc_obj) == Optional:
            doc_obj = self.design_doc_obj.get()
            if type(doc_obj) == JavadocComment:
                self.decla.setJavadocComment(dcs)
                return
        if not type(self.design_doc_obj) == BlockComment:
            block_comment = BlockComment(JString(dcs))
            self.design_doc_obj = block_comment
        body = self.decla.getBody().get()
        if len(body.getStatements()) > 0:
            firstStatement = body.getStatements().get(0)
        if dcs == '':
            firstStatement.removeComment()
        else:
            comment_lines = dcs.splitlines()
            indentation = find_indentation_string_after_newline(str(self.decla.toString()))
            formatted_comment = '\n' + '\n'.join((f'{indentation * 2}* {line}' for line in comment_lines)) + f'\n{indentation * 2}'
            firstStatement.setBlockComment(formatted_comment)

class JImportDef(ImportDef,JDecla):
        pass

def decla_list_2_string(dlist, str_spliter=' '):
    str_def = ''
    if dlist == None:
        return str_def
    for m in dlist:
        str_def += f'{str(m.toString())}{str_spliter}'
    if len(str_def) != 0:
        str_def = str_def[:-1]
    return str_def

class JClassDef(ClassDef, JDecla):

    def build(self):
        delca_type = str(type(self.decla))
        if delca_type == "<java class 'com.github.javaparser.ast.body.ClassOrInterfaceDeclaration'>":
            if self.decla.isInterface():
                self.delca_type = 'interface'
            else:
                self.delca_type = 'class'
        elif delca_type == "<java object 'com.github.javaparser.ast.body.AnnotationDeclaration'>":
            self.delca_type = 'annotation'
        elif delca_type == "<java class 'com.github.javaparser.ast.body.EnumDeclaration'>":
            self.delca_type = 'enum'
        else:
            self.delca_type = ''
        super().build()
        self.full_name = self.source.module_name + '.' + self.name
        self.build_signature()

    def build_design_doc_string(self):
        decla = self.decla
        self.design_doc_string = ''
        if hasattr(decla, 'hasJavaDocComment') and decla.hasJavaDocComment():
            str_doc = str(decla.getJavadocComment().get().toString())
            str_doc = remove_c_style_comments(str_doc)
            if startswith_action_key(str_doc):
                self.design_doc_string = str_doc
                self.design_doc_obj = decla.getJavadocComment()
        if self.design_doc_string == '':
            all_contained_comments = decla.getAllContainedComments()
            if len(all_contained_comments) > 0:
                comment = all_contained_comments[0]
                str_doc = str(comment.toString())
                str_doc = remove_c_style_comments(str_doc)
                if startswith_action_key(str_doc):
                    self.design_doc_string = str_doc
                    self.design_doc_obj = comment

    def build_signature(self):
        str_def = ''
        decla = self.decla
        if hasattr(decla, 'getModifiers'):
            str_def += decla_list_2_string(decla.getModifiers())
        str_def += f'{self.delca_type} '
        str_def += f'{self.name} '
        if hasattr(decla, 'getImplementedTypes'):
            str_def += decla_list_2_string(decla.getImplementedTypes())
        self.signature = str_def
        return str_def

def find_method_by_signature(class_decl, new_method):
    """
    根据方法签名在类中查找现有方法（考虑重载）。
    :param class_decl: ClassOrInterfaceDeclaration 对象
    :param new_method: MethodDeclaration 对象
    :return: MethodDeclaration 对象或 None
    """
    try:
        methods = class_decl.getMethodsByName(new_method.getNameAsString())
        for i in range(methods.size()):
            existing_method = methods.get(i)
            if parameters_match(existing_method.getParameters(), new_method.getParameters()):
                return existing_method
        return None
    except Exception as e:
        print(f'Error finding method by signature: {e}')
        return None

def parameters_match(existing_params, new_params):
    """
    比较两个参数列表是否匹配。
    :param existing_params: JavaParser 的 Parameter 列表
    :param new_params: JavaParser 的 Parameter 列表
    :return: 布尔值，表示参数是否匹配
    """
    if existing_params.size() != new_params.size():
        return False
    for i in range(existing_params.size()):
        existing_param = existing_params.get(i)
        new_param = new_params.get(i)
        if not existing_param.getType().equals(new_param.getType()):
            return False
    return True

def parse_code(new_code):
    from ai.d2c import JParserHelper
    imports_cu = None
    methods_cu = None
    code_parts = re.split('(import\\s+[\\w\\.]+\\s*;\\s*)', new_code)
    if len(code_parts) == 1:
        imports = ''
        methods = code_parts[0]
    else:
        imports = code_parts[0]
        methods = ''.join(code_parts[1:])
    has_class_definition = 'class ' in methods
    if has_class_definition:
        methods_cu = JParserHelper.parseCodeBlock(methods)
    else:
        wrapped_code = 'public class DummyClass { ' + new_code + ' }'
        methods_cu = JParserHelper.parseCodeBlock(wrapped_code)
    if imports:
        imports_cu = JParserHelper.parseCodeBlock(imports)
    return (imports_cu, methods_cu)

def find_indentation_string_after_newline(text: str):
    newline_index = text.find('\n')
    if newline_index == -1:
        return None
    after_newline = text[newline_index + 1:]
    indentation = ''
    for char in after_newline:
        if char == ' ' or char == '\t':
            indentation += char
        else:
            break
    return indentation